subroutine set_candidates( nat1, typ1, coords1, &
                           nat2, typ2, coords2, &
                           candidate1, candidate2 )

  !> @brief
  !! Subroutine to set candidate central atoms for structures 1 and 2.
  !!
  !! The decision is currently based simply on the total number of atoms,
  !! if the number of atoms is equal, then both candidates get a -1 value,
  !! otherwise the value of candidates is the atomic index of desired
  !! central atom.
  !!
  !! UPDATE NEEDED:
  !! candidate central atoms in 2 can only be of the same typ as in 1
  integer,                  intent(in) :: nat1
  integer, dimension(nat1), intent(in) :: typ1
  real, dimension(3,nat1),  intent(in) :: coords1
  integer,                  intent(in) :: nat2
  integer, dimension(nat2), intent(in) :: typ2
  real, dimension(3,nat2),  intent(in) :: coords2
  integer, dimension(nat1), intent(out) :: candidate1
  integer, dimension(nat2), intent(out) :: candidate2

  integer :: i, dnat

  candidate1(:) = 0
  candidate2(:) = 0
  !!
  !! some preprocessing
  dnat = nat2-nat1
  !!
  if( dnat .eq. 0 ) then
     !! nat1 = nat2

     !! value -1 indicates some special vector (geometrical center)
     candidate1(1) = -1

     !! value -1 indicates some special vector (geometrical center)
     candidate2(1) = -1


  elseif( dnat .gt. 0) then
     !! nat2 > nat1

     !! in struc 1 take the first atom
     candidate1(1) = 1

     !! in struc 2 take all atoms
     do i = 1, nat2
        candidate2(i) = i
     end do

  else
     write(*,*) 'error in set_candidate'
     stop
  endif



end subroutine set_candidates



subroutine select_rc( nat, coords, c_idx, rc )
  implicit none
  integer, intent(in) :: nat
  real, dimension(3,nat), intent(in) :: coords
  integer, intent(in) :: c_idx
  real, dimension(3), intent(out) :: rc

  if( c_idx .eq. 0 ) then
     write(*,*) 'ERROR in select_rc'
     return
  endif

  if( c_idx .eq. -1 ) then
     !! rc is geometric center
     rc = sum(coords(:,:),2)/nat
  else
     !! rc is vector of atom c_idx
     rc = coords(:,c_idx)
  endif

  return
end subroutine select_rc

